# -*- coding: utf-8 -*-

"""A library that provides a custome logger for the `iland.Api` object."""

import logging
import sys

#: A customer Python logging logger for the `iland.Api` object.
LOG = logging.getLogger('iland_sdk')
LOG.setLevel(logging.INFO)

ch = logging.StreamHandler(sys.__stdout__)
ch.setLevel(logging.INFO)

formatter = logging.Formatter(
    '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)

LOG.addHandler(ch)



# -*- coding: utf-8 -*-

"""A library that provides constants for the `iland.Api` object."""

#: Base URL to the iland cloud API.
BASE_URL = 'https://api.ilandcloud.com/iland-core/rest/v1'

#: Access token URL.
ACCESS_URL = \
    'https://ecs.ilandcloud.com/auth/realms/iland-core/tokens/grants/access'

#: Refresh token URL.
REFRESH_URL = \
    'https://ecs.ilandcloud.com/auth/realms/iland-core/tokens/refresh'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_iland_int
----------------------------------

Integration tests for the `iland` module.
"""

import time
import unittest

import iland
import iland.constant
from iland.exception import ApiException, UnauthorizedException

try:
    # only PyCharm w/ Python3 fails here without dot
    from apicreds import (CLIENT_ID,
                          CLIENT_SECRET,
                          USERNAME,
                          PASSWORD)
except ImportError:
    from .apicreds import (CLIENT_ID,
                           CLIENT_SECRET,
                           USERNAME,
                           PASSWORD)

VDC_UUID = \
    'res01.ilandcloud.com:urn:vcloud:vdc:a066325d-6be0-4733-8d9f-7687c36f4536'


@unittest.skipIf(not CLIENT_ID and not CLIENT_SECRET and not USERNAME and not
                 PASSWORD, "No credentials provided")
class TestIlandInt(unittest.TestCase):
    def setUp(self):
        self._api = iland.Api(client_id=CLIENT_ID,
                              client_secret=CLIENT_SECRET,
                              username=USERNAME,
                              password=PASSWORD,
                              base_url=iland.constant.BASE_URL)

    def tearDown(self):
        pass

    def test_get(self):
        user = self._api.get('/user/' + USERNAME)
        self.assertEqual(USERNAME, user.get('name'))
        self.assertTrue(len(user.keys()) > 5)

    def test_post(self):
        user_alert_emails = self._api.get('/user/' + USERNAME +
                                          '/alert-emails')
        self.assertTrue(len(user_alert_emails['emails']) >= 1)
        self.assertEquals(user_alert_emails['username'], USERNAME)

        old_user_alert_emails = user_alert_emails
        user_alert_emails = {'emails': ['test@iland.com', 'test2@iland.com'],
                             'username': USERNAME}
        self._api.post('/user/' + USERNAME + '/alert-emails',
                       user_alert_emails)

        self.assertTrue(len(user_alert_emails['emails']) >= 1)
        self.assertEquals(user_alert_emails['username'], USERNAME)

        self._api.post('/user/' + USERNAME + '/alert-emails',
                       old_user_alert_emails)
        self.assertEquals(len(user_alert_emails['emails']), 2)
        self.assertEquals(user_alert_emails['username'], USERNAME)

    def test_put_delete(self):
        vdc_uuid = VDC_UUID
        vdc_md = self._api.get('/vdc/' + vdc_uuid + '/metadata')

        self.assertEquals(len(vdc_md), 1)

        new_md = [{'type': 'string',
                   'value': 'B',
                   'access': 'READ_WRITE',
                   'key': 'AAA'}]
        new_md.extend(vdc_md)
        task = self._api.put('/vdc/' + vdc_uuid + '/metadata',
                             form_data=new_md)
        if task is None:
            tasks = self._api.get(
                '/task/res01.ilandcloud.com/entity/' + VDC_UUID + '/active')
            if len(tasks) > 0:
                task = tasks[0]
        while task is not None:
            task = self._api.get('/task/res01.ilandcloud.com/' + task[
                'task_id'])
            if task is not None and task['sychronized'] is True:
                break

        time.sleep(10)

        updated_md = self._api.get('/vdc/' + vdc_uuid + '/metadata')
        self.assertEquals(len(updated_md), 2)

        task = self._api.delete('/vdc/' + vdc_uuid + '/metadata/' + 'AAA')
        if task is None:
            tasks = self._api.get(
                '/task/res01.ilandcloud.com/entity/' + VDC_UUID + '/active')
            if len(tasks) > 0:
                task = tasks[0]
        while task is not None:
            task = self._api.get(
                '/task/res01.ilandcloud.com/' + task['task_id'])
            if task is not None and task['sychronized'] is True:
                break

        time.sleep(10)

        updated_md = self._api.get('/vdc/' + vdc_uuid + '/metadata')
        self.assertEquals(len(updated_md), 1)

    def test_refresh_token(self):
        self._api.login()
        self.assertIsNotNone(self._api.get_access_token())
        expires_in = self._api._token_expiration_time

        # force expires for tests.
        self._api._token_expiration_time = 0
        self._api.refresh_access_token()

        new_expires_in = self._api._token_expiration_time
        self.assertTrue(new_expires_in > expires_in)

    def test_unauthorized_errors(self):
        wrongCredsApi = iland.Api(client_id=CLIENT_ID,
                                  client_secret=CLIENT_SECRET,
                                  username='XXXX',
                                  password='XXXX',
                                  base_url=iland.constant.BASE_URL)
        with self.assertRaises(UnauthorizedException):
            wrongCredsApi.login()

        with self.assertRaises(UnauthorizedException):
            wrongCredsApi.get_access_token()

        with self.assertRaises(UnauthorizedException):
            wrongCredsApi.refresh_access_token()

    def test_api_errors(self):

        with self.assertRaises(ApiException):
            self._api.get('/doesnotexist')



#!/usr/bin/env python
# -*- coding: utf-8 -*-


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

from setuptools import find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['requests==2.9.1'],

test_requirements = ['bumpversion==0.5.3',
                     'wheel>=0.29.0',
                     'watchdog==0.8.3',
                     'flake8==2.5.4',
                     'tox==2.3.1',
                     'coverage==4.0.3',
                     'Sphinx==1.3.6',
                     'requests_mock==0.7.0'],

setup(
    name='iland-sdk',
    version='0.4.0',
    description="iland cloud Python SDK",
    long_description=readme + '\n\n' + history,
    author="iland Internet Solutions, Corp",
    author_email='devops@iland.com',
    url='https://github.com/ilanddev/python-sdk',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='iland cloud sdk',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



# -*- coding: utf-8 -*-

"""A library that provides a Python interface to the iland cloud API."""

import json
import time

import requests

from .constant import BASE_URL, ACCESS_URL, REFRESH_URL
from .log import LOG
from .exception import ApiException, UnauthorizedException


class Api(object):
    """A python interface into the iland cloud API
    """

    _client_id = None
    _client_secret = None
    _username = None
    _password = None
    _base_url = BASE_URL

    _token = None
    _token_expiration_time = None

    def __init__(self, client_id, client_secret, username, password,
                 base_url=None):
        """Instantiate a new iland.Api object.

        :param client_id: the client identifier
        :param client_secret: the client secret
        :param username: the iland cloud username
        :param password: the iland cloud password
        :param base_url: [optional] base url to contact the iland cloud API
        :return: Api Object
        """
        self._client_id = client_id
        self._client_secret = client_secret
        self._username = username
        self._password = password
        if base_url is not None:
            self._base_url = base_url
        else:
            self._base_url = BASE_URL

    def _get_access_token(self):

        if self._valid_token():
            return self._token

        LOG.info("SSO Request %s" % ACCESS_URL)

        params = {'client_id': self._client_id,
                  'client_secret': self._client_secret,
                  'username': self._username,
                  'password': self._password,
                  'grant_type': 'password'}
        r = requests.post(ACCESS_URL, data=params)
        json_payload = json.loads(r.content.decode('ascii'))
        if r.status_code not in [200, 201, 202]:
            raise UnauthorizedException(json_payload)
        self._token = json_payload
        self._set_token_expiration_time()
        return self._token

    def _set_token_expiration_time(self):
        time_buffer = 10
        self._token_expiration_time = \
            ((self._token['expires_in'] - time_buffer) * 1000) + \
            int(round(time.time() * 1000))

    def _refresh_token(self):
        if not self._valid_token():
            if self._token is not None:
                LOG.info("SSO Request %s" % REFRESH_URL)
                params = {'client_id': self._client_id,
                          'client_secret': self._client_secret,
                          'grant_type': 'refresh_token',
                          'refresh_token': self._token['refresh_token']
                          }
                r = requests.post(REFRESH_URL, data=params)
                json_payload = json.loads(r.content.decode('ascii'))
                if r.status_code not in [200, 201, 202]:
                    raise UnauthorizedException(json_payload)
                self._token = json_payload
                self._set_token_expiration_time()
            else:
                self._get_access_token()
        return self._token

    def _valid_token(self):
        if self._token is not None:
            return int(round(time.time() * 1000)) < self._token_expiration_time
        return False

    def _get_access_token_string(self):
        token_string = self._get_access_token()['access_token']
        return token_string

    def _do_request(self, rpath, verb='GET', form_data=None):
        self._refresh_token()
        data = None
        if form_data is not None:
            data = json.dumps(form_data, ensure_ascii=False).encode("UTF8")
        url = self._base_url + rpath

        LOG.info("Request %s rpath %s" % (verb, url))

        headers = {
            'Authorization': 'Bearer %s' % self._get_access_token_string(),
            'content-type': 'application/json'}
        if verb == 'GET':
            r = requests.get(url, headers=headers)
        elif verb == 'PUT':
            r = requests.put(url, data=data, headers=headers)
        elif verb == 'POST':
            r = requests.post(url, data=data, headers=headers)
        elif verb == 'DELETE':
            r = requests.delete(url, headers=headers)
        else:
            raise ApiException({'message': 'Unsupported HTTP verb %s' % verb})

        # iland cloud API prefix have to be ignored because they are here to
        # prevent JSON Hijacking
        json_obj = json.loads(r.content[5:].decode('UTF8'))
        if r.status_code not in [200, 201, 202, 204]:
            raise ApiException(json_obj)
        return json_obj

    def get(self, rpath):
        """ Perform a GET request agains the iland cloud API given its resource
        path.

        `iland.Api` will refresh the access token if non valid.

        :param rpath: the resource path as a Python builtin String object
        :raises: ApiException: API requests returns an error
        :raises: UnauthorizedException: credentials / grants invalids
        :return: a JSON Object or a list of JSON Objects.
        """
        return self._do_request(rpath)

    def put(self, rpath, form_data=None):
        """ Perform a PUT request agains the iland cloud API given its resource
        path.

        `iland.Api` will refresh the access token if non valid.

        :param rpath: the resource path as a Python builtin String object
        :param form_data: a Python builtin dict object
        :raises: ApiException: API requests returns an error
        :raises: UnauthorizedException: credentials / grants invalids
        :return: a JSON Object or a list of JSON Objects.
        """
        return self._do_request(rpath, verb='PUT', form_data=form_data)

    def post(self, rpath, form_data=None):
        """ Perform a POST request agains the iland cloud API given its resource
        path.

        `iland.Api` will refresh the access token if non valid.

        :param rpath: the resource path as a Python builtin String object
        :param form_data: a Python builtin dict object
        :raises: ApiException: API requests returns an error
        :raises: UnauthorizedException: credentials / grants invalids
        :return: a JSON Object or a list of JSON Objects.
        """
        return self._do_request(rpath, verb='POST', form_data=form_data)

    def delete(self, rpath):
        """ Perform a DELETE request agains the iland cloud API given its
        resource path.

        `iland.Api` will refresh the access token if non valid.

        :param rpath:  the resource path as a Python builtin String object
        :raises: ApiException: API requests returns an error
        :raises: UnauthorizedException: credentials / grants invalids
        :return: a JSON Object or a list of JSON Objects.
        """
        return self._do_request(rpath, verb='DELETE')

    def get_access_token(self):
        """ Returns the access token in use for this session.

        This method is exposed in case you interested in managing the token
        life cycle yourself. `iland.Api` will refresh the token on your
        behalf while performing queries.

        :raises: UnauthorizedException: credentials / grants invalids
        :return: JSON Object containing the actual access token
        """
        return self._get_access_token()

    def refresh_access_token(self):
        """ Refresh token if token is not valid: None or expired.

        This method is exposed in case you interested in managing the token
        life cycle yourself. `iland.Api` will refresh the token on your
        behalf while performing queries.

        :raises: UnauthorizedException: credentials / grants invalids
        :return: JSON Object containing the actual access token
        """
        return self._refresh_token()

    def login(self):
        """ Requests an access token.

        This method is exposed in case you interested in managing the token
        life cycle yourself. `iland.Api` will refresh the token on your
        behalf while performing queries.

        :raises: UnauthorizedException: credentials / grants invalids
        :return: JSON Object containing the actual access token
        """
        return self._get_access_token()
